@@ -12,7 +12,7 @@
 
 #include <ggv-postscript-view.h>
 
-BEGIN_GNOME_DECLS
+G_BEGIN_DECLS
  
 #define GGV_CONTROL_TYPE           (ggv_control_get_type ())
 #define GGV_CONTROL(o)             (GTK_CHECK_CAST ((o), GGV_CONTROL_TYPE, GgvControl))
@@ -21,9 +21,10 @@ BEGIN_GNOME_DECLS
 #define GGV_IS_CONTROL(o)          (GTK_CHECK_TYPE ((o), GGV_CONTROL_TYPE))
 #define GGV_IS_CONTROL_CLASS(k)    (GTK_CHECK_CLASS_TYPE ((k), GGV_CONTROL_TYPE))
 
-typedef struct _GgvControl         GgvControl;
-typedef struct _GgvControlClass    GgvControlClass;
-typedef struct _GgvControlPrivate  GgvControlPrivate;
+typedef struct _GgvControl              GgvControl;
+typedef struct _GgvControlClass         GgvControlClass;
+typedef struct _GgvControlPrivate       GgvControlPrivate;
+typedef struct _GgvControlClassPrivate  GgvControlClassPrivate;
 
 struct _GgvControl {
 	BonoboControl control;
@@ -33,6 +34,8 @@ struct _GgvControl {
 
 struct _GgvControlClass {
 	BonoboControlClass parent_class;
+
+	GgvControlClassPrivate *priv;
 };
 
 GtkType        ggv_control_get_type  (void);
@@ -40,6 +43,6 @@ GgvControl    *ggv_control_new       (GgvPostscriptView *ps_view);
 GgvControl    *ggv_control_construct (GgvControl *control,
 				      GgvPostscriptView *ps_view);
 
-END_GNOME_DECLS
+G_END_DECLS
 
-#endif _GGV_CONTROL
+#endif /* _GGV_CONTROL */
