@@ -4,7 +4,7 @@
  *
  * Author:  Jaka Mocnik  <jaka@gnu.org>
  *
- * Copyright (c) 2001 Free Software Foundation
+ * Copyright (c) 2001, 2002 Free Software Foundation
  */
 
 #include <config.h>
@@ -20,6 +20,7 @@
 #include <ggv-control.h>
 
 #include <gtkgs.h>
+#include <ps.h>
 #include <gsdefaults.h>
 #include <ggvutils.h>
 
@@ -50,10 +51,6 @@ struct _GgvPostscriptViewClassPrivate {
 	gchar          **size_paths;
 };
 
-enum {
-	LAST_SIGNAL
-};
-
 enum {
 	PROP_ANTIALIASING,
 	PROP_DEFAULT_ORIENTATION,
@@ -61,19 +58,23 @@ enum {
 	PROP_DEFAULT_SIZE,
 	PROP_OVERRIDE_SIZE,
 	PROP_RESPECT_EOF,
-	PROP_WATCH_FILE
+	PROP_WATCH_FILE,
+	PROP_WIDTH,
+	PROP_HEIGHT,
+	PROP_ORIENTATION,
+	PROP_PAGE,
+	PROP_PAGE_COUNT,
+	PROP_PAGE_NAMES,
+	PROP_TITLE,
+	PROP_STATUS
 };
 
 enum {
-	PROP_CONTROL_TITLE
+	PROP_CONTROL_TITLE,
 };
 
 static GdkCursor *pan_cursor = NULL;
 
-static guint ggv_postscript_view_signals [LAST_SIGNAL];
-
-#define PARENT_TYPE BONOBO_X_OBJECT_TYPE
-
 static BonoboObjectClass *ggv_postscript_view_parent_class;
 
 static gchar *orientation_paths[] = {
@@ -85,6 +86,106 @@ static gchar *orientation_paths[] = {
 	NULL
 };
 
+static void
+set_page_item_sensitivity(GgvPostscriptView *ps_view)
+{
+	GtkGS *gs = GTK_GS(ps_view->priv->gs);
+	gchar *prop_val;
+
+	prop_val = (gtk_gs_get_current_page(gs) >= gtk_gs_get_page_count(gs) - 1)?
+		"0":"1";
+	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/NextPage",
+								 "sensitive", prop_val, NULL);
+	prop_val = (gtk_gs_get_current_page(gs) <= 0 || !gs->structured_doc)?
+		"0":"1";
+	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/PrevPage",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/FirstPage",
+								 "sensitive", prop_val, NULL);
+	prop_val = (gtk_gs_get_current_page(gs) >= gtk_gs_get_page_count(gs) - 1 ||
+				!gs->structured_doc)?
+		"0":"1";
+	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/LastPage",
+								 "sensitive", prop_val, NULL);
+
+}
+
+static void
+notify_page_change(GgvPostscriptView *ps_view)
+{
+	BonoboArg *arg;
+
+	if(gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs)) < 0)
+		return;
+
+	arg = bonobo_arg_new(TC_CORBA_long);
+	*(CORBA_long *)arg->_value = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
+	bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
+										 "Bonobo/Property:change:page",
+										 arg, NULL);
+	bonobo_arg_release(arg);
+}
+
+void
+ggv_postscript_view_goto_page(GgvPostscriptView *ps_view, gint page)
+{
+	gint old_page = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
+
+	gtk_gs_goto_page(GTK_GS(ps_view->priv->gs), page);
+	if(gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs)) != old_page) {
+		notify_page_change(ps_view);
+		set_page_item_sensitivity(ps_view);
+	}
+}
+
+static void
+get_status(GgvPostscriptView *ps_view, BonoboArg *arg)
+{
+	GtkGS *gs = GTK_GS(ps_view->priv->gs);
+	gchar *status;
+
+	g_assert(arg->_type == TC_CORBA_string);
+	
+	status = "GGV control status...";
+
+	*(CORBA_string *)arg->_value = CORBA_string_dup(status);
+}
+
+static void
+get_title(GgvPostscriptView *ps_view, BonoboArg *arg)
+{
+	GtkGS *gs = GTK_GS(ps_view->priv->gs);
+	gchar *title;
+
+	g_assert(arg->_type == TC_CORBA_string);
+	
+	if(!gs->loaded)
+		title = "No document loaded.";
+	else if(!strcmp(gs->gs_filename, "-"))
+		title = "(stdin)";
+	else
+		title = gs->gs_filename;
+
+	*(CORBA_string *)arg->_value = CORBA_string_dup(title);
+}
+
+static void
+load_ps(GgvPostscriptView *ps_view, const gchar *fname)
+{
+	BonoboArg *arg;
+
+	gtk_gs_load(GTK_GS(ps_view->priv->gs), fname);
+	arg = bonobo_arg_new(TC_CORBA_string);
+	get_title(ps_view, arg);
+	bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
+										 "Bonobo/Property:change:title",
+										 arg, NULL);
+	bonobo_arg_release(arg);
+	gtk_gs_goto_page(GTK_GS(ps_view->priv->gs), 0);
+	set_page_item_sensitivity(ps_view);
+	/* TODO: notify about changes to status & sizes */
+}
+
 /*
  * Loads an postscript document from a Bonobo_Stream
  */
@@ -96,8 +197,9 @@ load_ps_from_stream (BonoboPersistStream *ps,
 					 CORBA_Environment *ev)
 {
 	GgvPostscriptView *ps_view;
-	Bonobo_Stream_iobuf  *buffer;
-	CORBA_long bytes_read, bytes_written;
+	guint8 *buffer;
+	CORBA_long len_read;
+	FILE *tmpfile;
 
 	g_return_if_fail (data != NULL);
 	g_return_if_fail (GGV_IS_POSTSCRIPT_VIEW (data));
@@ -112,12 +214,25 @@ load_ps_from_stream (BonoboPersistStream *ps,
 		ps_view->priv->tmp_name = g_malloc(256);
 	}
 	tmpnam(ps_view->priv->tmp_name);
-	buffer = Bonobo_Stream_iobuf__alloc();
-	Bonobo_Stream_copyTo(stream, ps_view->priv->tmp_name,
-						 -1, &bytes_read, &bytes_written, ev);
-	gtk_gs_load(GTK_GS(ps_view->priv->gs), ps_view->priv->tmp_name);
-	gtk_gs_set_page_size(GTK_GS(ps_view->priv->gs), -1, 0);
-	gtk_gs_goto_page(GTK_GS(ps_view->priv->gs), 0);
+	tmpfile = fopen(ps_view->priv->tmp_name, "w");
+	if(!tmpfile) {
+		return;
+	}
+	do {
+		buffer = bonobo_stream_client_read(stream, 32768, &len_read, ev);
+		if(buffer) {
+			if(len_read > 0) {
+				if(fwrite(buffer, 1, len_read, tmpfile) != len_read) {
+					g_free(buffer);
+					fclose(tmpfile);
+					return;
+				}
+			}
+			g_free(buffer);
+		}
+	} while(len_read != 32768);
+	fclose(tmpfile);
+	load_ps(ps_view, ps_view->priv->tmp_name);
 }
 
 /* 
@@ -138,7 +253,7 @@ view_button_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 			
 		gdk_window_get_pointer(widget->window, &wx, &wy, NULL);
 			
-		gtk_gs_start_scroll(ps_view->priv->gs);
+		gtk_gs_start_scroll(GTK_GS(ps_view->priv->gs));
 
 		ps_view->priv->pan = TRUE;
 		if(pan_cursor == NULL)
@@ -168,7 +283,7 @@ view_button_release_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 		ps_view->priv->pan = FALSE;
 		gdk_pointer_ungrab(GDK_CURRENT_TIME);
 		gtk_grab_remove(widget);
-		gtk_gs_end_scroll(ps_view->priv->gs);
+		gtk_gs_end_scroll(GTK_GS(ps_view->priv->gs));
 	}
 }
 
@@ -184,10 +299,10 @@ view_motion_cb(GtkWidget *widget, GdkEventMotion *event, gpointer data)
 
 	if(ps_view->priv->pan) {
 		gtk_gs_scroll(GTK_GS(ps_view->priv->gs),
-					  -event->x + ps_view->priv->prev_x,
-					  -event->y + ps_view->priv->prev_y);
-		ps_view->priv->prev_x = event->x;
-		ps_view->priv->prev_y = event->y;
+					  -(gint)event->x + ps_view->priv->prev_x,
+					  -(gint)event->y + ps_view->priv->prev_y);
+		ps_view->priv->prev_x = (gint)event->x;
+		ps_view->priv->prev_y = (gint)event->y;
 	}
 }
 
@@ -200,13 +315,14 @@ load_ps_from_file (BonoboPersistFile *pf, const CORBA_char *filename,
 				   CORBA_Environment *ev, void *data)
 {
 	GgvPostscriptView *ps_view;
+	BonoboArg *arg;
 
 	g_return_val_if_fail (data != NULL, -1);
 	g_return_val_if_fail (GGV_IS_POSTSCRIPT_VIEW (data), -1);
 
 	ps_view = GGV_POSTSCRIPT_VIEW (data);
 
-	gtk_gs_load(GTK_GS(ps_view->priv->gs), filename);
+	load_ps(ps_view, filename);
 
 	return 0;
 }
@@ -236,7 +352,7 @@ listener_Orientation_cb (BonoboUIComponent *uic, const char *path,
 	else if(!strcmp(path, "OrientationLandscape"))
 		orientation = GNOME_GGV_ORIENTATION_LANDSCAPE;
 	else if(!strcmp(path, "OrientationUpsideDown"))
-		orientation = GNOME_GGV_ORIENTATION_UPSIDE_DOWN;
+		orientation = GNOME_GGV_ORIENTATION_UPSIDEDOWN;
 	else if(!strcmp(path, "OrientationSeascape"))
 		orientation = GNOME_GGV_ORIENTATION_SEASCAPE;
 	else {
@@ -248,8 +364,8 @@ listener_Orientation_cb (BonoboUIComponent *uic, const char *path,
 	BONOBO_ARG_SET_GENERAL(arg, orientation, TC_GNOME_GGV_Orientation,
 						   GNOME_GGV_Orientation, NULL);
 
-	bonobo_property_bag_set_value(ps_view->priv->property_bag,
-								   "orientation", arg, NULL);
+	bonobo_pbclient_set_value(BONOBO_OBJREF(ps_view->priv->property_bag),
+							  "default_orientation", arg, NULL);
 
 	bonobo_arg_release(arg);
 }
@@ -304,8 +420,8 @@ listener_Toggle_cb (BonoboUIComponent *uic, const char *path,
 	BONOBO_ARG_SET_GENERAL(arg, flag, TC_CORBA_boolean,
 						   CORBA_boolean, NULL);
 
-	bonobo_property_bag_set_value(ps_view->priv->property_bag,
-								  prop, arg, NULL);
+	bonobo_pbclient_set_value(BONOBO_OBJREF(ps_view->priv->property_bag),
+							  prop, arg, NULL);
 
 	bonobo_arg_release(arg);
 }
@@ -337,29 +453,12 @@ listener_Size_cb(BonoboUIComponent *uic, const char *path,
 	BONOBO_ARG_SET_GENERAL(arg, size, TC_GNOME_GGV_Size,
 						   GNOME_GGV_Size, NULL);
 
-	bonobo_property_bag_set_value(ps_view->priv->property_bag,
-								  "size", arg, NULL);
+	bonobo_pbclient_set_value(BONOBO_OBJREF(ps_view->priv->property_bag),
+							  "default_size", arg, NULL);
 
 	bonobo_arg_release(arg);
 }
 
-static void
-set_page_item_sensitivity(GgvPostscriptView *ps_view)
-{
-	GtkGS *gs = GTK_GS(ps_view->priv->gs);
-	gchar *prop_val;
-
-	prop_val = (gtk_gs_get_current_page(gs) >= gtk_gs_get_page_count(gs) - 1)?
-		"0":"1";
-	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/NextPage",
-								 "sensitive", prop_val, NULL);
-	prop_val = (gtk_gs_get_current_page(gs) <= 0 || !gs->structured_doc)?
-		"0":"1";
-	bonobo_ui_component_set_prop(ps_view->priv->uic, "/commands/PrevPage",
-								 "sensitive", prop_val, NULL);
-}
-
-
 static void
 verb_NextPage_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 {
@@ -372,7 +471,7 @@ verb_NextPage_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 	ps_view = GGV_POSTSCRIPT_VIEW (user_data);
 	gs = GTK_GS(ps_view->priv->gs);
 
-	gtk_gs_goto_page(gs, gtk_gs_get_current_page(gs) + 1);
+	ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 
 	set_page_item_sensitivity(ps_view);
 }
@@ -387,28 +486,58 @@ verb_PrevPage_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 
 	ps_view = GGV_POSTSCRIPT_VIEW (user_data);
 
-	gtk_gs_goto_page(GTK_GS(ps_view->priv->gs),
-					 GTK_GS(ps_view->priv->gs)->current_page - 1);
+	ggv_postscript_view_goto_page(ps_view,
+								  GTK_GS(ps_view->priv->gs)->current_page - 1);
 
 	set_page_item_sensitivity(ps_view);
 }
 
+static void
+verb_LastPage_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
+{
+	GgvPostscriptView *ps_view;
+	GtkGS *gs;
+
+	g_return_if_fail (user_data != NULL);
+	g_return_if_fail (GGV_IS_POSTSCRIPT_VIEW (user_data));
+
+	ps_view = GGV_POSTSCRIPT_VIEW (user_data);
+	gs = GTK_GS(ps_view->priv->gs);
+
+	ggv_postscript_view_goto_page(ps_view, gtk_gs_get_page_count(gs) - 1);
+
+	set_page_item_sensitivity(ps_view);
+}
+
+static void
+verb_FirstPage_cb(BonoboUIComponent *uic, gpointer user_data, const char *cname)
+{
+	GgvPostscriptView *ps_view;
+	GtkGS *gs;
+
+	g_return_if_fail (user_data != NULL);
+	g_return_if_fail (GGV_IS_POSTSCRIPT_VIEW (user_data));
+
+	ps_view = GGV_POSTSCRIPT_VIEW (user_data);
+	gs = GTK_GS(ps_view->priv->gs);
+
+	ggv_postscript_view_goto_page(ps_view, 0);
+
+	set_page_item_sensitivity(ps_view);
+}
 static void
 sync_orientation_items(GgvPostscriptView *ps_view)
 {
-	CORBA_Environment ev;
 	gint i;
 	gboolean val, orient_state;
 
-	CORBA_exception_init(&ev);
-
 	val = gtk_gs_get_override_orientation(GTK_GS(ps_view->priv->gs));
 	bonobo_ui_component_set_prop(ps_view->priv->uic, orientation_paths[0],
 								 "state", val?"1":"0", NULL);
 	for(i = 1; orientation_paths[i] != NULL; i++) {
 		bonobo_ui_component_set_prop(ps_view->priv->uic, orientation_paths[i],
 									 "sensitive", val?"1":"0", NULL);
-		orient_state = i - 1 == gtk_gs_get_default_orientation(GTK_GS(ps_view->priv->gs));
+		orient_state = (i - 1 == gtk_gs_get_default_orientation(GTK_GS(ps_view->priv->gs)));
 		bonobo_ui_component_set_prop(ps_view->priv->uic, orientation_paths[i],
 									 "state", orient_state?"1":"0", NULL);
 	}
@@ -417,14 +546,11 @@ sync_orientation_items(GgvPostscriptView *ps_view)
 static void
 sync_size_items(GgvPostscriptView *ps_view)
 {
-	CORBA_Environment ev;
 	gint i;
 	gboolean val, size_state;
 	gchar **paths;
 
-	CORBA_exception_init(&ev);
-
-	paths = GGV_POSTSCRIPT_VIEW_CLASS(GTK_OBJECT(ps_view)->klass)->priv->size_paths;
+	paths = GGV_POSTSCRIPT_VIEW_CLASS(G_OBJECT_GET_CLASS(ps_view))->priv->size_paths;
 	val = gtk_gs_get_override_size(GTK_GS(ps_view->priv->gs));
 	bonobo_ui_component_set_prop(ps_view->priv->uic, paths[0],
 								 "state", val?"1":"0", NULL);
@@ -443,34 +569,29 @@ ggv_postscript_view_create_ui(GgvPostscriptView *ps_view)
 	static BonoboUIVerb ggv_postscript_view_verbs[] = {
 		BONOBO_UI_VERB ("NextPage", verb_NextPage_cb),
 		BONOBO_UI_VERB ("PrevPage", verb_PrevPage_cb),
+		BONOBO_UI_VERB ("LastPage", verb_LastPage_cb),
+		BONOBO_UI_VERB ("FirstPage", verb_FirstPage_cb),
 		BONOBO_UI_VERB_END
 	};
 	BonoboUINode *new_size_menus;
 	BonoboUINode *ui_root;
-	GgvPostscriptViewClass *klass = GGV_POSTSCRIPT_VIEW_CLASS(GTK_OBJECT(ps_view)->klass);
+	GgvPostscriptViewClass *klass = GGV_POSTSCRIPT_VIEW_CLASS(G_OBJECT_GET_CLASS(ps_view));
 	gint i;
 
 	g_return_if_fail(ps_view != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view));
 
 	/* Set up the UI from an XML file. */
-#if 1
-	ui_root = bonobo_ui_util_new_ui(ps_view->priv->uic, 
-									DATADIR "/gnome/ui/ggv-postscript-view-ui.xml",
-									DATADIR,
-									"GgvPostscriptView");
-	bonobo_ui_util_translate_ui(ui_root);
-	bonobo_ui_component_set_tree(ps_view->priv->uic, "/", ui_root, NULL);
-#else
 	bonobo_ui_util_set_ui(ps_view->priv->uic, DATADIR,
 						  "ggv-postscript-view-ui.xml", 
-						  "GgvPostscriptView");
-#endif
+						  "GGV", NULL);
 
 	new_size_menus = bonobo_ui_node_copy(klass->priv->size_menus, TRUE);
-	if(new_size_menus)
-		bonobo_ui_component_set_tree(ps_view->priv->uic, "/menu/View/Size",
+
+	if(new_size_menus) {
+		bonobo_ui_component_set_tree(ps_view->priv->uic, "/menu/ViewPlaceholder/View/Size/SizeList",
 									 new_size_menus, NULL);
+	}
 
 	bonobo_ui_component_add_listener(ps_view->priv->uic, "OrientationPortrait",
 									 listener_Orientation_cb, ps_view);
@@ -515,6 +636,119 @@ ggv_postscript_view_get_prop(BonoboPropertyBag *bag,
 	ps_view = GGV_POSTSCRIPT_VIEW(user_data);
 
 	switch(arg_id) {
+	case PROP_PAGE: {
+		g_assert(arg->_type == TC_CORBA_long);
+
+		*(CORBA_long *)arg->_value = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
+		break;
+	}
+	case PROP_PAGE_COUNT: {
+		g_assert(arg->_type == TC_CORBA_long);
+
+		*(CORBA_long *)arg->_value = gtk_gs_get_page_count(GTK_GS(ps_view->priv->gs));
+		break;
+	}
+	case PROP_PAGE_NAMES: {
+		GNOME_GGV_PageNameList *names;
+		GtkGS *gs;
+		int i;
+
+		g_assert(arg->_type == TC_GNOME_GGV_PageNameList);
+
+		names = GNOME_GGV_PageNameList__alloc();
+		names->_length = 0;
+		names->_buffer = NULL;
+		gs = GTK_GS(ps_view->priv->gs);
+		if(gs->loaded && gs->doc && gs->structured_doc) {
+			names->_length = gs->doc->numpages;
+			names->_buffer = CORBA_sequence_CORBA_string_allocbuf(names->_length);
+			for(i = 0; i < gs->doc->numpages; i++) {
+				names->_buffer[i] = CORBA_string_dup(gs->doc->pages[i].label);
+			}
+		}
+		CORBA_sequence_set_release(names, CORBA_TRUE);
+		arg->_value = names;
+		break;
+	}
+	case PROP_TITLE: {
+		g_assert(arg->_type == TC_CORBA_string);
+		get_title(ps_view, arg);
+		break;
+	}
+	case PROP_STATUS: {
+		g_assert(arg->_type == TC_CORBA_string);
+		get_status(ps_view, arg);
+		break;
+	}
+	case PROP_ORIENTATION: {
+		GNOME_GGV_Orientation orient;
+
+		g_assert(arg->_type == TC_GNOME_GGV_Orientation);
+
+		switch(gtk_gs_get_orientation(GTK_GS(ps_view->priv->gs))) {
+		case GTK_GS_ORIENTATION_PORTRAIT:
+			orient = GNOME_GGV_ORIENTATION_PORTRAIT;
+			break;
+		case GTK_GS_ORIENTATION_LANDSCAPE:
+			orient = GNOME_GGV_ORIENTATION_LANDSCAPE;
+			break;
+		case GTK_GS_ORIENTATION_UPSIDEDOWN:
+			orient = GNOME_GGV_ORIENTATION_UPSIDEDOWN;
+			break;
+		case GTK_GS_ORIENTATION_SEASCAPE:
+			orient = GNOME_GGV_ORIENTATION_SEASCAPE;
+			break;
+		default:
+			orient = GNOME_GGV_ORIENTATION_PORTRAIT;
+			break;
+		}
+		*(GNOME_GGV_Orientation *)arg->_value = orient;
+		break;
+	}
+	case PROP_WIDTH: {
+		CORBA_float w;
+		GtkGSOrientation orient;
+		GtkGS *gs = GTK_GS(ps_view->priv->gs);
+
+		g_assert(arg->_type == TC_CORBA_float);
+
+        orient = gtk_gs_get_orientation(gs);
+		switch(orient) {
+		case GTK_GS_ORIENTATION_PORTRAIT:
+        case GTK_GS_ORIENTATION_UPSIDEDOWN:
+			w = gs->urx - gs->llx;
+			break;
+        case GTK_GS_ORIENTATION_LANDSCAPE:
+        case GTK_GS_ORIENTATION_SEASCAPE:
+			w = gs->ury - gs->lly;
+			break;
+		}
+		w = MAX(w, 0);
+		*(CORBA_long *)arg->_value = w;
+		break;
+	}
+	case PROP_HEIGHT: {
+		CORBA_float h;
+		GtkGSOrientation orient;
+		GtkGS *gs = GTK_GS(ps_view->priv->gs);
+
+		g_assert(arg->_type == TC_CORBA_float);
+
+        orient = gtk_gs_get_orientation(gs);
+		switch(orient) {
+		case GTK_GS_ORIENTATION_PORTRAIT:
+        case GTK_GS_ORIENTATION_UPSIDEDOWN:
+			h = gs->ury - gs->lly;
+			break;
+        case GTK_GS_ORIENTATION_LANDSCAPE:
+        case GTK_GS_ORIENTATION_SEASCAPE:
+			h = gs->urx - gs->llx;
+			break;
+		}
+		h = MAX(h, 0);
+		*(CORBA_long *)arg->_value = h;
+		break;
+	}
 	case PROP_DEFAULT_ORIENTATION: {
 		g_assert(arg->_type == TC_GNOME_GGV_Orientation);
 
@@ -591,9 +825,14 @@ ggv_postscript_view_set_prop(BonoboPropertyBag *bag,
 
 	ps_view = GGV_POSTSCRIPT_VIEW(user_data);
 
-	klass = GGV_POSTSCRIPT_VIEW_CLASS(GTK_OBJECT(ps_view)->klass);
+	klass = GGV_POSTSCRIPT_VIEW_CLASS(G_OBJECT_GET_CLASS(ps_view));
 
 	switch(arg_id) {
+	case PROP_PAGE: {
+		g_assert(arg->_type == TC_CORBA_long);
+		ggv_postscript_view_goto_page(ps_view, *(CORBA_long *)arg->_value);
+		break;
+	}
 	case PROP_DEFAULT_ORIENTATION: {
 		GtkGSOrientation orient = GTK_GS_ORIENTATION_PORTRAIT;
 
@@ -608,7 +847,7 @@ ggv_postscript_view_set_prop(BonoboPropertyBag *bag,
 		case GNOME_GGV_ORIENTATION_LANDSCAPE:
 			orient = GTK_GS_ORIENTATION_LANDSCAPE;
 			break;
-		case GNOME_GGV_ORIENTATION_UPSIDE_DOWN:
+		case GNOME_GGV_ORIENTATION_UPSIDEDOWN:
 			orient = GTK_GS_ORIENTATION_UPSIDEDOWN;
 			break;
 		case GNOME_GGV_ORIENTATION_SEASCAPE:
@@ -619,7 +858,10 @@ ggv_postscript_view_set_prop(BonoboPropertyBag *bag,
 		}
 
 		gtk_gs_set_default_orientation(GTK_GS(ps_view->priv->gs), orient);
+
+		/*
 		sync_orientation_items(ps_view);
+		*/
 
 		break;
 	}
@@ -745,7 +987,8 @@ ggv_postscript_view_add_interfaces (GgvPostscriptView *ps_view,
 								 BONOBO_OBJECT (stream));
 
 	/* Interface Bonobo::PersistFile */
-	file = bonobo_persist_file_new (load_ps_from_file, NULL, ps_view);
+	file = bonobo_persist_file_new (load_ps_from_file, NULL,
+									"OAFIID:GNOME_GGV_PostscriptView", ps_view);
 	if (!file) {
 		bonobo_object_unref (BONOBO_OBJECT (to_aggregate));
 		return NULL;
@@ -798,7 +1041,7 @@ ggv_postscript_view_set_ui_container(GgvPostscriptView *ps_view,
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view));
 	g_return_if_fail(ui_container != CORBA_OBJECT_NIL);
 
-	bonobo_ui_component_set_container(ps_view->priv->uic, ui_container);
+	bonobo_ui_component_set_container(ps_view->priv->uic, ui_container, NULL);
 
 	ggv_postscript_view_create_ui(ps_view);
 }
@@ -809,7 +1052,7 @@ ggv_postscript_view_unset_ui_container(GgvPostscriptView *ps_view)
 	g_return_if_fail(ps_view != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view));
 
-	bonobo_ui_component_unset_container(ps_view->priv->uic);
+	bonobo_ui_component_unset_container(ps_view->priv->uic, NULL);
 }
 
 GtkWidget *
@@ -867,8 +1110,8 @@ ggv_postscript_view_zoom_to_fit(GgvPostscriptView *ps_view,
 }
 
 void
-ggv_postscript_view_set_orientation(GgvPostscriptView *ps_view,
-									GNOME_GGV_Orientation orientation)
+ggv_postscript_view_set_default_orientation(GgvPostscriptView *ps_view,
+											GNOME_GGV_Orientation orientation)
 {
 	BonoboArg *arg;
 
@@ -879,20 +1122,20 @@ ggv_postscript_view_set_orientation(GgvPostscriptView *ps_view,
 	BONOBO_ARG_SET_GENERAL(arg, orientation, TC_GNOME_GGV_Orientation,
 						   GNOME_GGV_Orientation, NULL);
 
-	bonobo_property_bag_set_value(ps_view->priv->property_bag,
-								  "orientation", arg, NULL);
+	bonobo_pbclient_set_value(BONOBO_OBJREF(ps_view->priv->property_bag),
+							  "default_orientation", arg, NULL);
 }
 
 GNOME_GGV_Orientation
-ggv_postscript_view_get_orientation(GgvPostscriptView *ps_view)
+ggv_postscript_view_get_default_orientation(GgvPostscriptView *ps_view)
 {
 	BonoboArg *arg;
 
 	g_return_val_if_fail(ps_view != NULL, 0);
 	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view), 0);
 
-	arg = bonobo_property_bag_get_value(ps_view->priv->property_bag,
-										"orientation", NULL);
+	arg = bonobo_pbclient_get_value(BONOBO_OBJREF(ps_view->priv->property_bag),
+									"default_orientation", NULL, NULL);
 	if(arg == NULL)
 		return 0;
 
@@ -901,7 +1144,7 @@ ggv_postscript_view_get_orientation(GgvPostscriptView *ps_view)
 }
 
 static void
-ggv_postscript_view_destroy(GtkObject *object)
+ggv_postscript_view_destroy(BonoboObject *object)
 {
 	GgvPostscriptView *ps_view;
 
@@ -926,11 +1169,12 @@ ggv_postscript_view_destroy(GtkObject *object)
 		bonobo_object_unref(BONOBO_OBJECT(ps_view->priv->uic));
 	ps_view->priv->uic = NULL;
 
-	GTK_OBJECT_CLASS(ggv_postscript_view_parent_class)->destroy(object);
+	if (BONOBO_OBJECT_CLASS (ggv_postscript_view_parent_class)->destroy)
+		BONOBO_OBJECT_CLASS(ggv_postscript_view_parent_class)->destroy(object);
 }
 
 static void
-ggv_postscript_view_finalize(GtkObject *object)
+ggv_postscript_view_finalize(GObject *object)
 {
 	GgvPostscriptView *ps_view;
 
@@ -941,13 +1185,14 @@ ggv_postscript_view_finalize(GtkObject *object)
 
 	g_free(ps_view->priv);
 
-	GTK_OBJECT_CLASS(ggv_postscript_view_parent_class)->finalize(object);
+	G_OBJECT_CLASS(ggv_postscript_view_parent_class)->finalize(object);
 }
 
 static void
 ggv_postscript_view_class_init(GgvPostscriptViewClass *klass)
 {
-	GtkObjectClass *object_class = (GtkObjectClass *)klass;
+	GObjectClass *object_class = (GObjectClass *)klass;
+	BonoboObjectClass *bonobo_object_class = (BonoboObjectClass *)klass;
 	gint n = gtk_gs_defaults_get_paper_count(), i;
 	BonoboUINode *placeholder;
 	BonoboUINode *item;
@@ -960,12 +1205,11 @@ ggv_postscript_view_class_init(GgvPostscriptViewClass *klass)
 	POA_GNOME_Ggv_PostscriptView__epv *epv;
 #endif
 
-	ggv_postscript_view_parent_class = gtk_type_class(PARENT_TYPE);
+	ggv_postscript_view_parent_class = g_type_class_peek_parent(klass);
 
 	/* init size menu template */
 	placeholder = bonobo_ui_node_new("placeholder");
 	bonobo_ui_node_set_attr(placeholder, "name", "SizeList");
-	bonobo_ui_node_set_attr(placeholder, "delimit", "top");
 	papersizes = gtk_gs_defaults_get_paper_sizes();
 	klass->priv->size_paths = g_new0(gchar *, n + 2);
 	for(i = 0; i < n; i++) {
@@ -985,9 +1229,7 @@ ggv_postscript_view_class_init(GgvPostscriptViewClass *klass)
 	klass->priv->size_paths[n + 1] = NULL;
 	klass->priv->size_menus = placeholder;
 
-	gtk_object_class_add_signals(object_class, ggv_postscript_view_signals, LAST_SIGNAL);
-
-	object_class->destroy = ggv_postscript_view_destroy;
+	bonobo_object_class->destroy = ggv_postscript_view_destroy;
 	object_class->finalize = ggv_postscript_view_finalize;
 	
 #if 0
@@ -1003,10 +1245,10 @@ ggv_postscript_view_init(GgvPostscriptView *ps_view)
 	ps_view->priv = g_new0(GgvPostscriptViewPrivate, 1);
 }
 
-BONOBO_X_TYPE_FUNC_FULL(GgvPostscriptView, 
-						GNOME_GGV_PostscriptView,
-						PARENT_TYPE,
-						ggv_postscript_view);
+BONOBO_TYPE_FUNC_FULL(GgvPostscriptView, 
+					  GNOME_GGV_PostscriptView,
+					  BONOBO_TYPE_OBJECT,
+					  ggv_postscript_view);
 
 static void
 property_control_get_prop(BonoboPropertyBag *bag,
@@ -1064,6 +1306,8 @@ property_control_get_cb(BonoboPropertyControl *property_control,
 #endif
 }
 
+
+
 GgvPostscriptView *
 ggv_postscript_view_construct(GgvPostscriptView *ps_view,
 							  GtkGS *gs, gboolean zoom_fit)
@@ -1081,7 +1325,7 @@ ggv_postscript_view_construct(GgvPostscriptView *ps_view,
 	ps_view->priv->gs = GTK_WIDGET(gs);
 	gtk_widget_set_events(GTK_WIDGET(gs), 
 						  GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK |
-						  GDK_POINTER_MOTION_MASK);
+						  GDK_POINTER_MOTION_MASK | GDK_KEY_PRESS_MASK);
 	gtk_signal_connect(GTK_OBJECT(gs), "button_press_event",
 					   GTK_SIGNAL_FUNC(view_button_press_cb), ps_view);
 	gtk_signal_connect(GTK_OBJECT(gs), "button_release_event",
@@ -1096,11 +1340,44 @@ ggv_postscript_view_construct(GgvPostscriptView *ps_view,
 														  ggv_postscript_view_set_prop,
 														  ps_view);
 
+
+	bonobo_property_bag_add(ps_view->priv->property_bag, "title",
+							PROP_TITLE, TC_CORBA_string, NULL,
+							_("Document title"), 
+							BONOBO_PROPERTY_READABLE);
+	bonobo_property_bag_add(ps_view->priv->property_bag, "status",
+							PROP_STATUS, TC_CORBA_string, NULL,
+							_("GGV control status"), 
+							BONOBO_PROPERTY_READABLE);
+	bonobo_property_bag_add(ps_view->priv->property_bag, "page_count",
+							PROP_PAGE_COUNT, TC_CORBA_long, NULL,
+							_("Number of pages"), 
+							BONOBO_PROPERTY_READABLE);
+	bonobo_property_bag_add(ps_view->priv->property_bag, "page_names",
+							PROP_PAGE_NAMES, TC_GNOME_GGV_PageNameList, NULL,
+							_("Page names"), 
+							BONOBO_PROPERTY_READABLE);
+	bonobo_property_bag_add(ps_view->priv->property_bag, "page",
+							PROP_PAGE, TC_CORBA_long, NULL,
+							_("Current page number"), 
+							BONOBO_PROPERTY_READABLE | BONOBO_PROPERTY_WRITEABLE);
+	bonobo_property_bag_add(ps_view->priv->property_bag, "width",
+							PROP_WIDTH, TC_CORBA_float, NULL,
+							_("Document width"), 
+							BONOBO_PROPERTY_READABLE);
+	bonobo_property_bag_add(ps_view->priv->property_bag, "height",
+							PROP_HEIGHT, TC_CORBA_float, NULL,
+							_("Document height"), 
+							BONOBO_PROPERTY_READABLE);
 	bonobo_property_bag_add(ps_view->priv->property_bag, "orientation",
+							PROP_ORIENTATION, TC_GNOME_GGV_Orientation, NULL,
+							_("Document orientation"), 
+							BONOBO_PROPERTY_READABLE);
+	bonobo_property_bag_add(ps_view->priv->property_bag, "default_orientation",
 							PROP_DEFAULT_ORIENTATION, TC_GNOME_GGV_Orientation, NULL,
 							_("Default orientation"), 
 							BONOBO_PROPERTY_READABLE | BONOBO_PROPERTY_WRITEABLE);
-	bonobo_property_bag_add(ps_view->priv->property_bag, "size",
+	bonobo_property_bag_add(ps_view->priv->property_bag, "default_size",
 							PROP_DEFAULT_SIZE, TC_GNOME_GGV_Size, NULL,
 							_("Default size"), 
 							BONOBO_PROPERTY_READABLE | BONOBO_PROPERTY_WRITEABLE);
@@ -1145,7 +1422,19 @@ ggv_postscript_view_new(GtkGS *gs, gboolean zoom_fit)
 	g_return_val_if_fail(gs != NULL, NULL);
 	g_return_val_if_fail(GTK_IS_GS(gs), NULL);
 
-	ps_view = gtk_type_new(ggv_postscript_view_get_type());
+	ps_view = g_object_new(GGV_POSTSCRIPT_VIEW_TYPE, NULL);
 
 	return ggv_postscript_view_construct(ps_view, gs, zoom_fit);
 }
+
+GtkAdjustment *
+ggv_postscript_view_get_hadj(GgvPostscriptView *ps_view)
+{
+	return GTK_GS(ps_view->priv->gs)->hadj;
+}
+
+GtkAdjustment *
+ggv_postscript_view_get_vadj(GgvPostscriptView *ps_view)
+{
+	return GTK_GS(ps_view->priv->gs)->vadj;
+}
