@@ -18,12 +18,11 @@
 
 #include <Ggv.h>
 
-BEGIN_GNOME_DECLS
- 
+G_BEGIN_DECLS
+
 #define GGV_POSTSCRIPT_VIEW_TYPE          (ggv_postscript_view_get_type ())
 #define GGV_POSTSCRIPT_VIEW(o)            (GTK_CHECK_CAST ((o), GGV_POSTSCRIPT_VIEW_TYPE, GgvPostscriptView))
 #define GGV_POSTSCRIPT_VIEW_CLASS(k)      (GTK_CHECK_CLASS_CAST((k), GGV_POSTSCRIPT_VIEW_TYPE, GgvPostscriptViewClass))
-
 #define GGV_IS_POSTSCRIPT_VIEW(o)         (GTK_CHECK_TYPE ((o), GGV_POSTSCRIPT_VIEW_TYPE))
 #define GGV_IS_POSTSCRIPT_VIEW_CLASS(k)   (GTK_CHECK_CLASS_TYPE ((k), GGV_POSTSCRIPT_VIEW_TYPE))
 
@@ -33,13 +32,13 @@ typedef struct _GgvPostscriptViewPrivate      GgvPostscriptViewPrivate;
 typedef struct _GgvPostscriptViewClassPrivate GgvPostscriptViewClassPrivate;
 
 struct _GgvPostscriptView {
-	BonoboXObject base;
+	BonoboObject base;
 
 	GgvPostscriptViewPrivate *priv;
 };
 
 struct _GgvPostscriptViewClass {
-	BonoboXObjectClass parent_class;
+	BonoboObjectClass parent_class;
 
 	POA_GNOME_GGV_PostscriptView__epv epv;
 
@@ -75,18 +74,25 @@ void  ggv_postscript_view_set_zoom        (GgvPostscriptView *image_view,
 BonoboObject *ggv_postscript_view_add_interfaces (GgvPostscriptView *ps_view,
 						  BonoboObject *to_aggregate);
 
+void ggv_postscript_view_goto_page(GgvPostscriptView *ps_view, gint page);
+
 /* Properties */
 void
-ggv_postscript_view_set_orientation (GgvPostscriptView *image_view,
-				     GNOME_GGV_Orientation interpolation);
+ggv_postscript_view_set_orientation (GgvPostscriptView *ps_view,
+				     GNOME_GGV_Orientation orientation);
 GNOME_GGV_Orientation
-ggv_postscript_view_get_orientation (GgvPostscriptView *image_view);
-void
-ggv_postscript_view_set_media        (GgvPostscriptView *image_view,
-				      GNOME_GGV_Media media);
-GNOME_GGV_Media
-ggv_postscript_view_get_media        (GgvPostscriptView *image_view);
+ggv_postscript_view_get_orientation (GgvPostscriptView *ps_view);
 
-END_GNOME_DECLS
+void
+ggv_postscript_view_set_size        (GgvPostscriptView *ps_view,
+				     GNOME_GGV_Size size);
+GNOME_GGV_Size
+ggv_postscript_view_get_size        (GgvPostscriptView *ps_view);
+
+GtkAdjustment *ggv_postscript_view_get_hadj(GgvPostscriptView *ps_view);
+GtkAdjustment *ggv_postscript_view_get_vadj(GgvPostscriptView *ps_view);
+
+
+G_END_DECLS
 
 #endif /* _GGV_POSTSCRIPT_VIEW_H_ */
