@@ -17,9 +17,10 @@
 #include <errno.h>
 
 #include <gnome.h>
-#include <liboaf/liboaf.h>
 
-#include <bonobo.h>
+#include <bonobo/bonobo-ui-main.h> 
+
+#include <bonobo/bonobo-generic-factory.h>
 
 #include <ggv-control.h>
 #if 0
@@ -46,10 +47,10 @@ ggv_postscript_viewer_factory (BonoboGenericFactory *this, const char *oaf_iid,
 
 	if (!strcmp (oaf_iid, "OAFIID:GNOME_GGV_Control"))
 		retval = BONOBO_OBJECT (ggv_control_new (ps_view));
-	/*
+#if 0
 	else if (!strcmp (oaf_iid, "OAFIID:GNOME_GGV_Embeddable"))
 		retval = BONOBO_OBJECT (ggv_embeddable_new (ps_view));
-	*/
+#endif
 	else if(!strcmp(oaf_iid, "OAFIID:GNOME_GGV_PostscriptView")) {
 		retval = BONOBO_OBJECT(ps_view);
 		bonobo_object_ref (BONOBO_OBJECT (ps_view));
@@ -64,49 +65,5 @@ ggv_postscript_viewer_factory (BonoboGenericFactory *this, const char *oaf_iid,
 	return retval;
 }
 
-static void
-init_ggv_postscript_viewer_factory (void)
-{
-	BonoboGenericFactory *factory;
-
-	factory = bonobo_generic_factory_new_multi (
-		"OAFIID:GNOME_GGV_Factory",
-		ggv_postscript_viewer_factory, NULL);
-
-	bonobo_running_context_auto_exit_unref (
-		BONOBO_OBJECT (factory));
-}
-
-static void
-init_server_factory (int argc, char **argv)
-{
-	CORBA_Environment ev;
-	CORBA_exception_init (&ev);
-
-        gnome_init_with_popt_table ("ggv-postscript-viewer", VERSION,
-				    argc, argv,
-				    oaf_popt_options, 0, NULL); 
-	oaf_init (argc, argv);
-
-	if (bonobo_init (CORBA_OBJECT_NIL, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL) == FALSE)
-		g_error (_("I could not initialize Bonobo"));
-
-	CORBA_exception_free (&ev);
-}
-
-int
-main (int argc, char *argv [])
-{
-	bindtextdomain (PACKAGE, GNOMELOCALEDIR);
-	textdomain (PACKAGE);
-
-        gconf_init (argc, argv, NULL);
-
-	init_server_factory (argc, argv);
-
-	init_ggv_postscript_viewer_factory ();
-
-	bonobo_main ();
-	
-	return 0;
-}
+BONOBO_ACTIVATION_FACTORY("OAFIID:GNOME_GGV_Factory", "ggv-postscript-viewer", 
+			  VERSION, ggv_postscript_viewer_factory, NULL);
