@@ -112,10 +112,7 @@ The DONE message indicates that ghostscript has finished processing.
 #include <signal.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkprivate.h>
-#ifdef HAVE_XINERAMA
-#  include <X11/extensions/Xinerama.h>
-#  include <gdk/gdkx.h>
-#endif /* HAVE_XINERAMA */
+#include <gdk/gdkx.h>
 #include <X11/Intrinsic.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -130,6 +127,9 @@ The DONE message indicates that ghostscript has finished processing.
 #include "ps.h"
 #include "gsdefaults.h"
 
+//#include "ggv-marshal.h"
+
+
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
 #endif
@@ -170,7 +170,6 @@ static void gtk_gs_emit_error_msg (GtkGS *gs, const gchar *msg);
 static void gtk_gs_defaults_changed (GConfClient* client, guint cnxn_id,
                                      GConfEntry *entry, gpointer user_data);
 static void send_ps(GtkGS *gs, long begin, unsigned int len, gboolean close);
-static void gtk_gs_set_adjustments(GtkGS *gs, GtkAdjustment *hadj, GtkAdjustment *vadj);
 static void set_up_page (GtkGS *gs);
 static void close_pipe (int p[2]);
 static void interpreter_failed (GtkGS *gs);
@@ -190,15 +189,37 @@ static GtkGSClass *gs_class = NULL;
 static gint gtk_gs_signals[LAST_SIGNAL] = { 0 };
 
 /* Static, private functions */
+
 static void
-gtk_gs_marshaller(GtkObject *object, GtkSignalFunc func,
-                  gpointer func_data, GtkArg *args)
+ggv_marshaller_VOID__POINTER(GClosure     *closure,
+				  GValue       *return_value,
+                                  guint         n_param_values,
+                                  const GValue *param_values,
+                                  gpointer      invocation_hint,
+                                  gpointer      marshal_data)
 {
-        InterpreterMessageSignal rfunc;
+  typedef void (*GMarshalFunc_VOID__POINTER) (gpointer     data1,
+                                              gpointer     arg_1,
+                                              gpointer     data2);
+  register GMarshalFunc_VOID__POINTER callback;
+  register GCClosure *cc = (GCClosure*) closure;
+  register gpointer data1, data2;
 
-        rfunc = (InterpreterMessageSignal)func;
+  g_return_if_fail (n_param_values == 2);
 
-        (*rfunc)(object, GTK_VALUE_POINTER (args[0]), func_data);
+  if (G_CCLOSURE_SWAP_DATA (closure)) {
+          data1 = closure->data;
+          data2 = g_value_peek_pointer (param_values + 0);
+  }
+  else {
+          data1 = g_value_peek_pointer (param_values + 0);
+          data2 = closure->data;
+  }
+  callback = (GMarshalFunc_VOID__POINTER) (marshal_data ? marshal_data : cc->callback);
+
+  callback (data1,
+            g_value_get_pointer (param_values + 1),
+            data2);
 }
 
 static void
@@ -276,48 +297,42 @@ static void
 gtk_gs_class_init (GtkGSClass *klass)
 {
         GtkObjectClass *object_class;
+        GObjectClass  *gobject_class;
         GtkWidgetClass *widget_class;
 
         object_class    = (GtkObjectClass *) klass;
+        gobject_class   = (GObjectClass *)klass;
         widget_class    = (GtkWidgetClass *) klass;
         parent_class    = gtk_type_class(gtk_widget_get_type());
         gs_class = klass;
 
-	gtk_gs_signals[INTERPRETER_MESSAGE] = gtk_signal_new(
-                       "interpreter_message", GTK_RUN_LAST, object_class->type,
+        gtk_gs_signals[INTERPRETER_MESSAGE] = g_signal_new(
+                       "interpreter_message",
+                       G_TYPE_FROM_CLASS(object_class),
+                       G_SIGNAL_RUN_LAST,
                        GTK_SIGNAL_OFFSET(GtkGSClass, interpreter_message),
-                       gtk_gs_marshaller, GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
-	gtk_gs_signals[DOCUMENT_CHANGED] = gtk_signal_new(
-                       "document_changed", GTK_RUN_LAST, object_class->type,
-                       GTK_SIGNAL_OFFSET(GtkGSClass, document_changed),
-                       gtk_signal_default_marshaller, GTK_TYPE_NONE, 0);
-        gtk_object_class_add_signals(object_class, gtk_gs_signals, LAST_SIGNAL);
+                       NULL,
+                       NULL,
+                       ggv_marshaller_VOID__POINTER,
+                       G_TYPE_NONE,
+                       1, G_TYPE_POINTER);
 
         object_class->destroy = gtk_gs_destroy;
 
         widget_class->realize       = gtk_gs_realize;
-        widget_class->unrealize     = gtk_gs_unrealize;
         widget_class->size_request  = gtk_gs_size_request;
         widget_class->size_allocate = gtk_gs_size_allocate;
-        widget_class->set_scroll_adjustments_signal =
-                gtk_signal_new ("set_scroll_adjustments",
-                                GTK_RUN_LAST,
-                                object_class->type,
-                                GTK_SIGNAL_OFFSET(GtkGSClass, set_scroll_adjustments),
-                                gtk_marshal_NONE__POINTER_POINTER,
-                                GTK_TYPE_NONE, 2, GTK_TYPE_ADJUSTMENT, GTK_TYPE_ADJUSTMENT);
 
         /* Create atoms */
         klass->gs_atom        = gdk_atom_intern ("GHOSTVIEW", FALSE);
         klass->gs_colors_atom = gdk_atom_intern ("GHOSTVIEW_COLORS", FALSE);
-        klass->next_atom      = gdk_atom_intern ("NEXT", FALSE);
+        klass->next_atom      = gdk_atom_intern ("NEXT", TRUE);
         klass->page_atom      = gdk_atom_intern ("PAGE", FALSE);
         klass->done_atom      = gdk_atom_intern ("DONE", FALSE);
         klass->string_atom    = gdk_atom_intern ("STRING", FALSE);
 
         /* a default handler for "interpreter_message" signal */
         klass->interpreter_message = gtk_gs_interpreter_message;
-        klass->set_scroll_adjustments = gtk_gs_set_adjustments;
         gtk_gs_defaults_load();
 }
 
@@ -359,7 +374,6 @@ gtk_gs_defaults_changed(GConfClient* client, guint cnxn_id,
 static void
 gtk_gs_interpreter_message(GtkGS *gs, gchar *msg, gpointer user_data)
 {
-        /* g_print("interpreter message: %s\n", msg); */
         g_free(msg);
 }
 
@@ -432,11 +446,6 @@ gtk_gs_destroy (GtkObject *object)
                 gs->gconf_notify_id = -1;
         }
 
-        if(gs->hadj)
-                gtk_object_unref (GTK_OBJECT (gs->hadj));
-        if(gs->vadj)
-                gtk_object_unref (GTK_OBJECT (gs->vadj));
-
         if (GTK_OBJECT_CLASS (parent_class)->destroy)
                 (* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
@@ -523,7 +532,7 @@ gtk_gs_realize (GtkWidget *widget)
         attributes.wclass      = GDK_INPUT_OUTPUT;
         attributes.visual      = gtk_widget_get_visual (widget);
         attributes.colormap    = gtk_widget_get_colormap (widget);
-        attributes.event_mask  = gtk_widget_get_events (widget);
+        attributes.event_mask  = gtk_widget_get_events (widget) | GDK_EXPOSURE_MASK;
         attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
 
         widget->window = gdk_window_new (widget->parent->window, &attributes, attributes_mask);
@@ -537,7 +546,6 @@ gtk_gs_realize (GtkWidget *widget)
 
         gs->pstarget = gdk_window_new (widget->window, &attributes, attributes_mask);
         gdk_window_set_user_data (gs->pstarget, widget);
-        /* gdk_window_show (gs->pstarget); */
         gdk_window_clear (gs->pstarget);
         gtk_style_set_background (widget->style, gs->pstarget, GTK_STATE_ACTIVE);
         gs->psgc = gdk_gc_new(gs->pstarget);
@@ -552,8 +560,8 @@ gtk_gs_realize (GtkWidget *widget)
 		gtk_gs_munge_adjustments (gs);
         }
 
-        gtk_signal_connect(GTK_OBJECT(widget), "event",
-                           GTK_SIGNAL_FUNC(gtk_gs_widget_event), gs);
+        g_signal_connect(GTK_OBJECT(widget), "event",
+                         G_CALLBACK(gtk_gs_widget_event), gs);
 
         gtk_gs_goto_page(gs, gs->current_page);
 }
@@ -617,13 +625,12 @@ gtk_gs_size_allocate (GtkWidget     *widget,
                 event.y = allocation->y;
                 event.width = allocation->width;
                 event.height = allocation->height;
-
                 gtk_widget_event (widget, (GdkEvent*) &event);
         }
         /* 
          * update the adjustment if necessary (ie. a resize);
          */
-        if ((gs->width > 0) && (gs->height > 0) && GTK_WIDGET_REALIZED(gs)) {
+        if ((gs->width > 0) && (gs->height >0) && GTK_WIDGET_REALIZED(gs)) {
 		gtk_gs_munge_adjustments (gs);
         }
 
@@ -636,20 +643,14 @@ gtk_gs_widget_event(GtkWidget *widget,
                     gpointer data)
 {
         GtkGS *gs = (GtkGS *)data;
-
 	if (event->type != GDK_CLIENT_EVENT) 
                 return FALSE;
 
         /* the first long is the window to communicate with gs,
          only if event if client_event */
-
         gs->message_window = event->client.data.l[0];
-        /* g_print("Message window %ld\n", gs->message_window); */
 
 	if (event->client.message_type == gs_class->page_atom) {
-                /*
-                  g_print("Interpreter completed: busy -> FALSE\n");
-                */
                 gs->busy = FALSE;
 	}
         return TRUE;
@@ -717,7 +718,7 @@ send_ps(GtkGS *gs, long begin, unsigned int len, gboolean close)
         struct record_list *ps_new;
         
         if (gs->interpreter_input < 0) {
-                g_print ("no pipe to gs\nerror in send_ps\n");
+                g_critical("No pipe to gs: error in send_ps().");
                 return;
         }
 
@@ -743,7 +744,8 @@ send_ps(GtkGS *gs, long begin, unsigned int len, gboolean close)
                                       GDK_INPUT_WRITE,
                                       input,
                                       gs);
-        } else {
+        }
+        else {
                 struct record_list *p = gs->ps_input;
                 while (p->next != NULL) {
                         p = p->next;
@@ -794,7 +796,7 @@ set_up_page (GtkGS *gs)
 { 
         guint orientation;
         char               buf[1024];
-        GdkPixmapPrivate  *pprivate;
+        GdkPixmap  *pprivate;
         GdkColormap *colormap;
         GdkGC *fill;
         GdkColor white = {0, 0xFFFF, 0xFFFF, 0xFFFF}; /* pixel, r, g, b */
@@ -813,7 +815,7 @@ set_up_page (GtkGS *gs)
                 gdk_flush(); 
 
                 if (gs->bpixmap) {
-                        gdk_pixmap_unref(gs->bpixmap);
+                        gdk_drawable_unref(gs->bpixmap);
                         gs->bpixmap = NULL;
                 }
                 gs->bpixmap = gdk_pixmap_new(gs->pstarget, gs->width, gs->height, -1);
@@ -841,9 +843,10 @@ set_up_page (GtkGS *gs)
          */
         savelocale = setlocale (LC_NUMERIC, "C");
 #endif
-        pprivate = (GdkPixmapPrivate *) gs->bpixmap;
+        pprivate = (GdkPixmap *) gs->bpixmap;
+
         snprintf(buf, 1024, "%ld %d %d %d %d %d %f %f %d %d %d %d",
-                 (pprivate ? pprivate->xwindow : 0L),
+                 gdk_x11_drawable_get_xid(pprivate), 
                  orientation * 90,
                  gs->llx, gs->lly,
                  gs->urx, gs->ury,
@@ -854,8 +857,6 @@ set_up_page (GtkGS *gs)
                  gs->right_margin,
                  gs->top_margin);
 
-        /*        g_print("Sent to GS: %s\n", buf); */
-
 #ifdef HAVE_SETLOCALE
         setlocale (LC_NUMERIC, savelocale);
 #endif
@@ -866,6 +867,7 @@ set_up_page (GtkGS *gs)
                             GDK_PROP_MODE_REPLACE,
                             buf,
                             strlen(buf));
+
         gdk_flush(); 
 }
 
@@ -881,8 +883,6 @@ close_pipe(int p[2])
 static gboolean
 is_interpreter_ready(GtkGS *gs)
 {
-        /*        g_print("Is interpreter ready pid %d busy %d %lx [%s]",  
-                  gs->interpreter_pid,gs->busy, gs->ps_input,gs->ps_input); */
         return (gs->interpreter_pid != -1 &&
                 !gs->busy &&
                 gs->ps_input == NULL);
@@ -913,7 +913,8 @@ output (gpointer data, gint source, GdkInputCondition condition)
                         interpreter_failed(gs);
                         return;
                 }
-        } else if (source == gs->interpreter_err) {
+        }
+        else if (source == gs->interpreter_err) {
                 bytes = read(gs->interpreter_err, buf, MAX_BUFSIZE);
                 if (bytes == 0) { /* EOF occurred */
                         close(gs->interpreter_err);
@@ -954,9 +955,8 @@ input (gpointer data, gint source, GdkInputCondition condition)
                         }
                         /* Have to seek at the beginning of each section */
                         if (gs->ps_input && gs->ps_input->seek_needed) {
-
-                                        fseek(gs->ps_input->fp,
-                                              gs->ps_input->begin, SEEK_SET);
+                                fseek(gs->ps_input->fp,
+                                      gs->ps_input->begin, SEEK_SET);
                                 gs->ps_input->seek_needed = FALSE;
                                 gs->bytes_left = gs->ps_input->len;
                         }
@@ -966,12 +966,14 @@ input (gpointer data, gint source, GdkInputCondition condition)
                                         fread(gs->input_buffer,
                                               sizeof (char), MAX_BUFSIZE,
                                               gs->ps_input->fp);
-                        } else if (gs->bytes_left > 0) {
+                        }
+                        else if (gs->bytes_left > 0) {
                                 gs->buffer_bytes_left =
                                         fread(gs->input_buffer,
                                               sizeof (char), gs->bytes_left,
                                               gs->ps_input->fp);
-                        } else {
+                        }
+                        else {
                                 gs->buffer_bytes_left = 0;
                         }
                         if (gs->bytes_left > 0 && gs->buffer_bytes_left == 0) {
@@ -992,11 +994,13 @@ input (gpointer data, gint source, GdkInputCondition condition)
                                 gtk_gs_emit_error_msg(gs, g_strdup(_("Broken pipe.")));
                                 broken_pipe = FALSE;
                                 interpreter_failed(gs);
-                        } else if (bytes_written == -1) {
+                        }
+                        else if (bytes_written == -1) {
                                 if ((errno != EWOULDBLOCK) && (errno != EAGAIN)) {
                                         interpreter_failed(gs);	/* Something bad happened */
                                 }
-                        } else {
+                        }
+                        else {
                                 gs->buffer_bytes_left -= bytes_written;
                                 gs->input_buffer_ptr += bytes_written;
                         }
@@ -1022,8 +1026,8 @@ start_interpreter (GtkGS *gs)
 	int std_out[2]; /* pipe from interp stdout */
 	int std_err[2]; /* pipe from interp stderr */
 
-#define NUM_ARGS       100
-#define NUM_GS_ARGS    (NUM_ARGS - 20)
+#define NUM_ARGS    100
+#define NUM_GS_ARGS (NUM_ARGS - 20)
 #define NUM_ALPHA_ARGS 10
 
         char *argv[NUM_ARGS], *dir, *gv_env;
@@ -1074,7 +1078,7 @@ start_interpreter (GtkGS *gs)
 
         if (!gs->reading_from_pipe && !gs->send_filename_to_gs) {
                 if (pipe (std_in) == -1) {
-                        g_print("Unable to open pipe to ghostview\n");
+                        g_critical ("Unable to open pipe to Ghostscript.");
                         return -1;
                 }
         }
@@ -1122,13 +1126,15 @@ start_interpreter (GtkGS *gs)
                         }
                 }
 
-                gv_env = g_strdup_printf ("GHOSTVIEW=%ld",
-                                          ((GdkWindowPrivate*)(gs->pstarget))->xwindow);
+                 gv_env = g_strdup_printf ("GHOSTVIEW=%ld",
+                                          gdk_x11_drawable_get_xid(gs->pstarget) );
                 putenv (gv_env);
+
+
                 /* change to directory where the input file is. This helps
                  * with postscript-files which include other files using
                  * a relative path */
-                dir = g_dirname(gs->gs_filename);
+                dir = g_path_get_dirname(gs->gs_filename);
                 chdir(dir);
                 g_free(dir);
 
@@ -1137,9 +1143,9 @@ start_interpreter (GtkGS *gs)
                 /* Notify error */
                 g_print("Unable to execute [%s]\n", argv[0]);
                 g_strfreev(gs_args);
+                g_free(gv_env);
                 if(alpha_args)
                         g_strfreev(alpha_args);
-                g_free(gv_env);
 		_exit(1);
 		break;
 	default: /* parent */
@@ -1225,7 +1231,6 @@ check_filecompressed (GtkGS *gs)
 	FILE *file;
 	gchar buf[1024];
 	gchar *filename, *filename_unc, *filename_err, *cmdline;
-        gint fd;
         const gchar *cmd;
 
 	cmd = NULL;
@@ -1248,22 +1253,8 @@ check_filecompressed (GtkGS *gs)
 
 	/* do the decompression */
 	filename = ggv_quote_filename (gs->gs_filename);
-	filename_unc = g_strconcat (g_get_tmp_dir(), "/uncXXXXXX", NULL);
-        fd = mkstemp (filename_unc);
-        if (fd < 0) {
-                g_free (filename_unc);
-                return filename;
-        }
-        close (fd);
-	filename_err = g_strconcat (g_get_tmp_dir(), "/errXXXXXX", NULL);
-        fd = mkstemp (filename_err);
-        if (fd < 0) {
-                g_free (filename_unc);
-                g_free (filename_err);
-                return filename;
-        }
-        close (fd);
-                
+	filename_unc = tempnam (g_get_tmp_dir(), "unc");
+	filename_err = tempnam (g_get_tmp_dir(), "err");
 	cmdline = g_strdup_printf ("%s %s >%s 2>%s", cmd,
 				   filename, filename_unc, filename_err);
 	if ((system (cmdline) == 0)
@@ -1307,7 +1298,6 @@ check_pdf (GtkGS *gs)
 {
 	FILE *file;
 	gchar buf[1024], *filename;
-        gint fd;
 
 	/* use uncompressed file as input if necessary */
 	filename = (gs->gs_filename_unc ? gs->gs_filename_unc
@@ -1317,24 +1307,10 @@ check_pdf (GtkGS *gs)
 	    && (fread (buf, sizeof(char), 5, file) == 5)
 	    && (strncmp (buf, "%PDF-", 5) == 0)) {
 		/* we found a PDF file */
-		gchar *fname, *filename_dsc, *filename_err, *cmd, *cmdline; 
-                filename_dsc = g_strconcat (g_get_tmp_dir(), "/dscXXXXXX", NULL);
-                fd = mkstemp (filename_dsc);
-                if (fd < 0) {
-                        g_free (filename_dsc);
-                        return gs->gs_filename;
-                }
-                close (fd);
-                filename_err = g_strconcat (g_get_tmp_dir(), "/errXXXXXX", NULL);
-                fd = mkstemp (filename_err);
-                if (fd < 0) {
-                        g_free (filename_dsc);
-                        g_free (filename_err);
-                        return gs->gs_filename;
-                }
-                close (fd);
-                
+		gchar *fname, *filename_dsc, *filename_err, *cmd, *cmdline;
 		fname = ggv_quote_filename (filename);
+		filename_dsc = tempnam (g_get_tmp_dir(), "dsc");
+		filename_err = tempnam (g_get_tmp_dir(), "err");
 		cmd = g_strdup_printf (gtk_gs_defaults_get_scan_pdf_cmd(),
                                        fname, filename_dsc);
 		g_free (fname);
@@ -1377,41 +1353,13 @@ check_pdf (GtkGS *gs)
 static float
 compute_xdpi(void)
 {
-#ifndef HAVE_XINERAMA
         return 25.4 * gdk_screen_width()/gdk_screen_width_mm();
-#else
-        Display *dpy;
-        dpy = (Display *) GDK_DISPLAY ();
-        if (XineramaIsActive (dpy)) {
-                int num_heads;
-                XineramaScreenInfo *head_info;
-                head_info = (XineramaScreenInfo *) XineramaQueryScreens (dpy, &num_heads);
-                /* fake it with dimensions of the first head for now */
-                return 25.4 * head_info[0].width/gdk_screen_width_mm();
-        } else {
-                return 25.4 * gdk_screen_width()/gdk_screen_width_mm();
-        }
-#endif /* HAVE_XINERAMA */
 }
- 
+
 static float
 compute_ydpi(void)
 {
-#ifndef HAVE_XINERAMA
         return 25.4 * gdk_screen_height()/gdk_screen_height_mm();
-#else
-        Display *dpy;
-        dpy = (Display *) GDK_DISPLAY ();
-        if (XineramaIsActive (dpy)) {
-                int num_heads;
-                XineramaScreenInfo *head_info;
-                head_info = (XineramaScreenInfo *) XineramaQueryScreens (dpy, &num_heads);
-                /* fake it with dimensions of the first head for now */
-                return 25.4 * head_info[0].height/gdk_screen_height_mm();
-        } else {
-                return 25.4 * gdk_screen_height()/gdk_screen_height_mm();
-        }
-#endif /* HAVE_XINERAMA */
 }
 
 /* Compute new size of window, sets xdpi and ydpi if necessary.
@@ -1448,7 +1396,6 @@ compute_size(GtkGS *gs)
         change = (new_width != gs->width * gs->zoom_factor) || (new_height != gs->height * gs->zoom_factor);
         gs->width = (gint) (new_width * gs->zoom_factor);
         gs->height = (gint) (new_height * gs->zoom_factor);
-
         if(GTK_WIDGET_REALIZED(gs)) {
                 if(!gs->loaded) {
                         if(gdk_window_is_visible(gs->pstarget))
@@ -1483,23 +1430,31 @@ gtk_gs_enable_interpreter (GtkGS *gs)
 
 /* publicly accessible functions */
 
-guint
-gtk_gs_get_type (void)
+GType gtk_gs_get_type (void)
 {
-        static guint gs_type = 0;
-
-        if (!gs_type) {
-                GtkTypeInfo gs_info = { "GtkGS",
-                                        sizeof (GtkGS),
+        static GType gs_type = 0;
+        if (!gs_type)
+        {
+                GTypeInfo gs_info = {
                                         sizeof (GtkGSClass),
-                                        (GtkClassInitFunc) gtk_gs_class_init,
-                                        (GtkObjectInitFunc) gtk_gs_init,
-                                        (GtkArgSetFunc) NULL,
-                                        (GtkArgGetFunc) NULL };
-                gs_type = gtk_type_unique (gtk_widget_get_type (), &gs_info);
-        }
+                                        (GBaseInitFunc) NULL,
+                                        (GBaseFinalizeFunc) NULL,
+                                        (GClassInitFunc) gtk_gs_class_init,
+                                        (GClassFinalizeFunc) NULL,
+                                        NULL, /* class_data */
+                                        sizeof (GtkGS),
+                                        0, /* n_preallocs */
+                                        (GtkObjectInitFunc) gtk_gs_init
+                                        };
 
+                gs_type = g_type_register_static(gtk_widget_get_type (),
+                                                "GtkGS",
+                                                 &gs_info,
+                                                 0);
+        }
         return gs_type;
+
+
 }
 
 GtkWidget *
@@ -1524,10 +1479,10 @@ gtk_gs_new (GtkAdjustment *hadj, GtkAdjustment *vadj)
         vadj->value = 0.0;
         vadj->page_size = 1.0;
 
-        gtk_signal_connect (GTK_OBJECT (hadj), "value_changed",
+        g_signal_connect (GTK_OBJECT (hadj), "value_changed",
                             (GtkSignalFunc) gtk_gs_value_adjustment_changed,
                             (gpointer) gs);
-        gtk_signal_connect (GTK_OBJECT (vadj), "value_changed",
+        g_signal_connect (GTK_OBJECT (vadj), "value_changed",
                             (GtkSignalFunc) gtk_gs_value_adjustment_changed,
                             (gpointer) gs);
 
@@ -1567,17 +1522,22 @@ gtk_gs_center_page(GtkGS *gs)
                 gdk_draw_rectangle(gs->pstarget, gs->psgc, FALSE,
                                    gs->scroll_start_x, gs->scroll_start_y,
                                    gs->scroll_width, gs->scroll_height);
-        gdk_window_move (gs->pstarget,
-                         (gs->widget.allocation.width - gs->width)/2,
-                         (gs->widget.allocation.height - gs->height)/2);
+        gdk_window_move(gs->pstarget,
+                        (gs->widget.allocation.width - gs->width)/2,
+                        (gs->widget.allocation.height - gs->height)/2);
+        while(gtk_events_pending()) {
+                gtk_main_iteration();
+        }
         if(gs->scroll_start_x != -1 && gs->scroll_start_y != -1)
                 gdk_draw_rectangle(gs->pstarget, gs->psgc, FALSE,
                                    gs->scroll_start_x, gs->scroll_start_y,
                                    gs->scroll_width, gs->scroll_height);
         gs->hadj->page_size = ((gfloat) gs->widget.allocation.width)/gs->width;
+        gs->hadj->page_size = MIN(gs->hadj->page_size, 1.0);
         gs->vadj->page_size = ((gfloat) gs->widget.allocation.height)/gs->height;
-        gs->hadj->value = (gs->hadj->upper - gs->hadj->lower)/2;
-        gs->vadj->value = (gs->vadj->upper - gs->vadj->lower)/2;
+        gs->vadj->page_size = MIN(gs->vadj->page_size, 1.0);
+        gs->hadj->value = 0.5 - gs->hadj->page_size/2;
+        gs->vadj->value = 0.5 - gs->vadj->page_size/2;
         gtk_adjustment_changed (gs->hadj);
         gtk_adjustment_changed (gs->vadj);
 }
@@ -1732,6 +1692,7 @@ gtk_gs_load (GtkGS *gs, const gchar *fname)
                 stat(fname, &sbuf);
                 gs->mtime = sbuf.st_mtime;
   	}
+        gtk_gs_set_page_size(gs, -1, gs->current_page);
 	gtk_widget_queue_resize (&(gs->widget));
         gs->loaded = TRUE;
 
@@ -1750,39 +1711,35 @@ gboolean
 gtk_gs_next_page(GtkGS *gs)
 {
 	XEvent event;
-	GdkWindowPrivate *pw;
 	
         g_return_val_if_fail(gs != NULL, FALSE);
         g_return_val_if_fail(GTK_IS_GS(gs), FALSE);
 
 	if (gs->interpreter_pid == 0) { /* no interpreter active */ 
-                /* g_print ("pid = 0\n"); */
+                 g_print ("pid = 0\n"); 
 		return FALSE;
         }
 
 	if (gs->busy) { /* interpreter is busy */
-                /* g_print ("int busy\n"); */
+                 g_print ("int busy\n"); 
 		return FALSE;
         }
 
 	gs->busy = TRUE;
-	pw = (GdkWindowPrivate *)gs->widget.window;
-
-        /*
-        g_print("Sending NEXT message to %ld\n", gs->message_window);
-        */
+ 
+        /* g_print("Sending NEXT message to %ld\n", gs->message_window);*/
+        
 	event.xclient.type          = ClientMessage;
-	event.xclient.display       = gdk_display;
+  	event.xclient.display       = gdk_display;
 	event.xclient.window        = gs->message_window;
-	event.xclient.message_type  = gs_class->next_atom;
+	event.xclient.message_type  = gdk_x11_atom_to_xatom(gs_class->next_atom);
 	event.xclient.format        = 32;
-        /*
-        g_print ("sending event.");
-        */
-	gdk_send_xevent(gs->message_window, FALSE, 0, &event);
-        /*
-        g_print ("..sent\n");
-        */
+        
+        /* g_print ("sending event."); */
+        gdk_send_xevent(gs->message_window, FALSE, 0, &event);     
+ 
+        /* g_print ("..sent\n"); */
+              
         gdk_flush();
 	return TRUE;
 }
@@ -1900,8 +1857,7 @@ gtk_gs_set_page_size(GtkGS *gs, gint new_pagesize, gint pageid)
         g_return_val_if_fail(GTK_IS_GS(gs), FALSE);
 
         if(new_pagesize == -1) {
-                if(gs->default_size > 0)
-                        new_pagesize = gs->default_size;
+                new_pagesize = gs->default_size;
                 if (!gs->override_size && gs->doc) {
                         /* If we have a document:
                            We use -- the page size (if specified)
@@ -1938,7 +1894,7 @@ gtk_gs_set_page_size(GtkGS *gs, gint new_pagesize, gint pageid)
                 new_urx = papersizes[new_pagesize].width;
                 new_ury = papersizes[new_pagesize].height;
         }
-        else if ((gs->doc->epsf && !gs->override_size) || new_pagesize == -1) { /* epsf or bbox */
+        else if ((gs->doc->epsf || new_pagesize == -1)) { /* epsf or bbox */
                 if((gs->doc->pages) &&
                    (gs->doc->pages[pageid].boundingbox[URX] > gs->doc->pages[pageid].boundingbox[LLX]) &&
                    (gs->doc->pages[pageid].boundingbox[URY] > gs->doc->pages[pageid].boundingbox[LLY])) {
@@ -1970,11 +1926,6 @@ gtk_gs_set_page_size(GtkGS *gs, gint new_pagesize, gint pageid)
                 }
         }
 
-        if(new_urx <= new_llx)
-                new_urx = papersizes[8].width;
-        if(new_ury <= new_lly)
-                new_ury = papersizes[8].height;
-
         /* If bounding box changed, setup for new size. */
                 /* gtk_gs_disable_interpreter (gs); */
         if ((new_llx != gs->llx) || (new_lly != gs->lly) ||
@@ -2311,6 +2262,7 @@ void
 gtk_gs_start_scroll(GtkGS *gs)
 {
         gint x, y, w, h;
+        GdkRectangle *rect; 
 
         if(!GTK_WIDGET_REALIZED(gs) || !gs->show_scroll_rect)
                 return;
@@ -2320,6 +2272,7 @@ gtk_gs_start_scroll(GtkGS *gs)
         gs->scroll_start_y = MAX(-y, 0);
         gs->scroll_width = MIN(gs->widget.allocation.width - 1, w - 1);
         gs->scroll_height = MIN(gs->widget.allocation.height - 1, h - 1);
+
         gdk_draw_rectangle(gs->pstarget, gs->psgc, FALSE,
                            gs->scroll_start_x, gs->scroll_start_y,
                            gs->scroll_width, gs->scroll_height);
@@ -2351,55 +2304,140 @@ gboolean gtk_gs_get_show_scroll_rect(GtkGS *gs)
         return gs->show_scroll_rect;
 }
 
-void
-gtk_gs_set_adjustments (GtkGS         *gs,
-                        GtkAdjustment *hadj,
-                        GtkAdjustment *vadj)
+gboolean 
+gtk_gs_scroll_to_edge(GtkGS *gs, GtkPositionType vertical, GtkPositionType horizontal)
 {
-  g_return_if_fail (gs != NULL);
-  g_return_if_fail (GTK_IS_GS (gs));
-  if (hadj)
-    g_return_if_fail (GTK_IS_ADJUSTMENT (hadj));
-  else
-    hadj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 1.0, 0.0, 0.0, 1.0));
-  if (vadj)
-    g_return_if_fail (GTK_IS_ADJUSTMENT (vadj));
-  else
-    vadj = GTK_ADJUSTMENT (gtk_adjustment_new (0.0, 0.0, 1.0, 0.0, 0.0, 1.0));
-  
-  if (gs->hadj && (gs->hadj != hadj))
-    {
-      gtk_signal_disconnect_by_data (GTK_OBJECT (gs->hadj), gs);
-      gtk_object_unref (GTK_OBJECT (gs->hadj));
-    }
-  
-  if (gs->vadj && (gs->vadj != vadj))
-    {
-      gtk_signal_disconnect_by_data (GTK_OBJECT (gs->vadj), gs);
-      gtk_object_unref (GTK_OBJECT (gs->vadj));
-    }
-  
-  if (gs->hadj != hadj)
-    {
-      gs->hadj = hadj;
-      gtk_object_ref (GTK_OBJECT (gs->hadj));
-      gtk_object_sink (GTK_OBJECT (gs->hadj));
-      
-      gtk_signal_connect (GTK_OBJECT (hadj), "value_changed",
-                          (GtkSignalFunc) gtk_gs_value_adjustment_changed,
-                          (gpointer) gs);
+        g_return_val_if_fail(gs != NULL, FALSE);
+        g_return_val_if_fail(GTK_IS_GS(gs), FALSE);
+
+        switch (vertical) {
+        case GTK_POS_BOTTOM:
+                gs->vadj->value = gs->vadj->lower;
+                gtk_adjustment_value_changed(gs->vadj);
+                break;
+        case GTK_POS_TOP:
+                gs->vadj->value = gs->vadj->upper - gs->vadj->page_size;
+                gtk_adjustment_value_changed(gs->vadj);
+                break;
+        default:
+                g_assert(0); /* Illegal parameter error */
+        }
 
 
-    }
+        switch (vertical) {
+        case GTK_POS_BOTTOM:
+                gs->hadj->value = gs->hadj->lower;
+                gtk_adjustment_value_changed(gs->hadj);
+                break;
+        case GTK_POS_TOP:
+                gs->hadj->value = gs->hadj->upper - gs->hadj->page_size;
+                gtk_adjustment_value_changed(gs->hadj);
+                break;
+        default:
+                g_assert(0); /* Illegal parameter error */
+        }
+        
+        return TRUE;
+}
+
+gboolean 
+gtk_gs_scroll_step(GtkGS *gs, GtkScrollType direction, gboolean dowrap)
+{
+        GtkAdjustment *MainAdj; /* We will move this adjustment */
+        GtkAdjustment *SecoAdj; /* And this _only_ if we can't move MainAdj (ie. we're edge)
+                                   and there is wrapping */
+
+        gboolean MoveHorizontal = TRUE; /* Positive if we move horizontal */
+        gboolean DirectionFlag = TRUE;  /* Positive if we move towards upper */
+        g_return_val_if_fail(gs != NULL, FALSE);
+        g_return_val_if_fail(GTK_IS_GS(gs), FALSE);
+
+#define EPSILON 0.00005
+
+#define CHECK_THERE_IS_NO_LOWER_SPACE(adj) \
+        ((adj)->value - (EPSILON) <= (adj)->lower)
+#define CHECK_THERE_IS_NO_UPPER_SPACE(adj) \
+        ((adj)->value + (EPSILON) >= (adj)->upper - (adj)->page_size)
+
+#define CHECK_THERE_IS_NO_SPACE_FOR_STEP(adj,dir) \
+        (dir?CHECK_THERE_IS_NO_UPPER_SPACE(adj):CHECK_THERE_IS_NO_LOWER_SPACE(adj))
+
+        /* To make code more readable, we make a macro */
+#define ADVANCE_TOWARDS_LOWER(adj) \
+        (adj->value -= 0.75 * (adj->page_size))
+#define ADVANCE_TOWARDS_UPPER(adj) \
+        (adj->value += 0.75 * (adj->page_size))
+
+#define ADVANCE_STEP(adj,dir) \
+        (dir?ADVANCE_TOWARDS_UPPER(adj):ADVANCE_TOWARDS_LOWER(adj))
+
+#define MOVE_TO_LOWER_EDGE(adj) \
+        (adj->value = adj->lower)
+#define MOVE_TO_UPPER_EDGE(adj) \
+        (adj->value = adj->upper - adj->page_size)
+        
+        /* if upper is 1 goto upper, otherwise to lower */
+#define MOVE_TO_EDGE(adj,upper) (upper?MOVE_TO_UPPER_EDGE(adj):MOVE_TO_LOWER_EDGE(adj))
+
+        /* These variables make our life easier */
+        switch (direction) {
+        case GTK_SCROLL_STEP_RIGHT:
+                MoveHorizontal = TRUE;
+                DirectionFlag = TRUE;
+                break;
+        case GTK_SCROLL_STEP_LEFT:
+                MoveHorizontal = TRUE;
+                DirectionFlag = FALSE;
+                break;
+        case GTK_SCROLL_STEP_DOWN:
+                MoveHorizontal = FALSE;
+                DirectionFlag = TRUE;
+                break;
+        case GTK_SCROLL_STEP_UP:
+                MoveHorizontal = FALSE;
+                DirectionFlag = FALSE;
+                break;
+        default:
+                g_warning("Illegal scroll step direction.");
+        }
+    
+        if (MoveHorizontal) {
+                MainAdj = gs->hadj;
+                SecoAdj = gs->vadj;
+        }
+        else {
+                MainAdj = gs->vadj;
+                SecoAdj = gs->hadj;
+        }
+
+        if (CHECK_THERE_IS_NO_SPACE_FOR_STEP(MainAdj, DirectionFlag)) {
+                if (!dowrap)
+                        return FALSE;
+                /* Move in the oposite axis */
+                if (CHECK_THERE_IS_NO_SPACE_FOR_STEP(SecoAdj, DirectionFlag)) {
+                        /* there is no place to move, we need a new page */
+                        return FALSE;
+                }
+                ADVANCE_STEP(SecoAdj,DirectionFlag);
+    
+                if (CHECK_THERE_IS_NO_SPACE_FOR_STEP(SecoAdj,DirectionFlag)) {
+                        /* We move it too far, lets move it to the edge */
+                        MOVE_TO_EDGE(SecoAdj,DirectionFlag);
+                }
+                /* now move to edge (other axis) in oposite direction */
+                MOVE_TO_EDGE(MainAdj,!DirectionFlag);
+                gtk_adjustment_value_changed(SecoAdj);
+                return TRUE;
+        }
   
-  if (gs->vadj != vadj)
-    {
-      gs->vadj = vadj;
-      gtk_object_ref (GTK_OBJECT (gs->vadj));
-      gtk_object_sink (GTK_OBJECT (gs->vadj));
-      
-      gtk_signal_connect (GTK_OBJECT (vadj), "value_changed",
-                          (GtkSignalFunc) gtk_gs_value_adjustment_changed,
-                          (gpointer) gs);
-    }
+        /* Now we know we can move in the direction sought */  
+        ADVANCE_STEP(MainAdj,DirectionFlag);
+  
+        if (CHECK_THERE_IS_NO_SPACE_FOR_STEP(MainAdj,DirectionFlag)) {
+                /* We move it too far, lets move it to the edge */
+                MOVE_TO_EDGE(MainAdj,DirectionFlag);
+        }
+        gtk_adjustment_value_changed(MainAdj);
+  
+        return TRUE;
 }
