@@ -41,8 +41,10 @@ struct _GtkGS
 {
         GtkWidget widget;                    /* the main widget */
 	GdkWindow *pstarget;                  /* the window passed to gv
-                                              * it is a child of widget...
-                                              */
+                                               * it is a child of widget...
+                                               */
+        GtkAdjustment *hadj, *vadj;
+
         GdkGC *psgc;
         gint scroll_start_x, scroll_start_y;
         gint scroll_width, scroll_height;
@@ -53,8 +55,10 @@ struct _GtkGS
 
         long       message_window;  /* Used by ghostview to receive messages from app */
         
+#if 0
 	GtkAdjustment* hadj;
 	GtkAdjustment* vadj;
+#endif
 
 	int        disable_start;		/* Can the interpreter be started? */
 	pid_t      interpreter_pid;		/* PID of interpreter, -1 if none  */
@@ -148,7 +152,7 @@ struct _GtkGSPaperSize
         gint width, height;
 };
 
-guint      gtk_gs_get_type (void);
+GType      gtk_gs_get_type (void);
 
 GtkWidget *gtk_gs_new_from_file(GtkAdjustment *hadj, GtkAdjustment *vadj, gchar *fname);
 GtkWidget *gtk_gs_new(GtkAdjustment *hadj, GtkAdjustment *vadj);
@@ -157,6 +161,8 @@ gboolean   gtk_gs_load (GtkGS *gs, const gchar *fname);
 /* control functions */
 void         gtk_gs_center_page (GtkGS *gs);
 void         gtk_gs_scroll (GtkGS *gs, gint, gint);
+gboolean     gtk_gs_scroll_step(GtkGS *gs, GtkScrollType direction, gboolean dowrap);
+gboolean     gtk_gs_scroll_to_edge(GtkGS *gs, GtkPositionType vertical, GtkPositionType horizontal);
 gboolean     gtk_gs_next_page(GtkGS *gs);
 gboolean     gtk_gs_prev_page(GtkGS *gs);
 gboolean     gtk_gs_goto_page(GtkGS *gs, gint);
@@ -198,8 +204,6 @@ void         gtk_gs_end_scroll(GtkGS *gs);
 /* utility functions */
 gint     gtk_gs_get_size_index(gchar *string, GtkGSPaperSize *size);
 
-GConfClient *gtk_gs_get_gconf_client(void);
-
 #define GTK_GS_IS_COMPRESSED(gs)       (gs->gs_filename_unc != NULL)
 #define GTK_GS_GET_PS_FILE(gs)         (GTK_GS_IS_COMPRESSED(gs) ? \
                                         gs->gs_filename_unc : \
